class Plate
     attr_reader :name, :ingredient_quantities, :ingredients
     
     @@food = {
         'fried egg' => EggMilkAndDerivatives.new('fried egg', [14.1, 0.0, 19.5]),
         'milk'      => EggMilkAndDerivatives.new('milk', [3.3, 4.8, 3.2]),
         'yogurt'    => EggMilkAndDerivatives.new('yogurt', [3.8, 4.9, 3.8]),
         'pork'      => MeatAndDerivatives.new('pork', [21.5, 0.0, 6.3]),
         'beef'      => MeatAndDerivatives.new('beef', [21.1, 0.0, 3.1]),
         'chicken'   => MeatAndDerivatives.new('chicken', [20.6, 0.0, 5.6]),
         'cod'       => Fish.new('cod', [17.7, 0.0, 0.4]),
         'tuna'      => Fish.new('tuna', [21.5, 0.0, 15.5]),
         'salmon'    => Fish.new('salmon', [19.9, 0.0, 13.6]),
         'olive oil' => OilsAndSweets.new('olive oil', [0.0, 0.2, 99.6]),
         'butter'    => OilsAndSweets.new('butter', [0.7, 0.0, 83.2]),
         'chocolate' => OilsAndSweets.new('chocolate', [5.3, 47.0, 30.0]),
         'sugar'     => CarbohydrateRich.new('sugar', [0.0, 99.8, 0.0]),
         'rice'      => CarbohydrateRich.new('rice', [6.8, 77.7, 0.6]),
         'lentils'   => CarbohydrateRich.new('lentils', [23.5, 52.0, 1.4]),
         'potatoe'   => CarbohydrateRich.new('potatoe', [2.0, 15.4, 0.1]),
         'tomato'    => Vegetables.new('tomato', [1.0, 3.5, 0.2]),
         'onion'     => Vegetables.new('onion', [1.3, 5.8, 0.3]),
         'pumpkin'   => Vegetables.new('pumpkin', [1.1, 4.8, 0.1]),
         'apple'     => Fruits.new('apple', [0.3, 12.4, 0.4]),
         'banana'    => Fruits.new('banana', [1.2, 21.4, 0.2]),
         'pear'      => Fruits.new('pear', [0.5, 12.7, 0.3])
     }

     # Getter for the food hash
     # @return [Hash<String, Food>] Hash mapping names to food instances
     def self.food
        @@food
     end
     
     def initialize(name, &block)
        @name = name
        @ingredients = []
        @ingredient_quantities = []
        
        if block_given?
            if block.arity == 1
                yield self
            else
                instance_eval(&block)
            end
        end
     end
     
     def add_ingredient(name, quantity)
          ingredient = @@food[name.strip.downcase]
          raise( ArgumentError, "Invalid ingredient: #{name}" ) unless !ingredient.nil?
          
          # Add the ingredient to the list
          @ingredients << ingredient
          
          # Little convenient lambda to not repeat code below
          # Given a key of quantity, it checks that the value is Numeric.
          # If it is numeric, its value times the multiplier is added to
          # the @ingredient_quantities array
          check_and_add_amount = ->(key, multiplier = 1.0) {
           amount = quantity[key]
           raise( ArgumentError, "Invalid amount type: '#{amount}' must be of type Numeric" ) unless amount.is_a? Numeric
           @ingredient_quantities << amount * multiplier
          }
          
          # Check what metric is used for the quantity and add
          if quantity.has_key?(:grams)
            check_and_add_amount.call(:grams, 1.0)
          
          elsif quantity.has_key?(:ounces)
            check_and_add_amount.call(:ounces, 28.35)
          
          elsif quantity.has_key?(:liters)
            check_and_add_amount.call(:liters, 1.0)
          
          elsif quantity.has_key?(:mililiters)
            check_and_add_amount.call(:mililiters, 1.0 / 1000.0)
          
          elsif quantity.has_key?(:portion)
               # Get the string and convert it to an array
               value = quantity[:portion].split(" ")
               
               # Get the quantity and the string
               amount = value[0].to_f
               str = value[1..-1].join(" ").downcase
               
               case str
                    when "cup", "cups"
                         amount *= 1.0
                    when "teaspoon", "teaspoons"
                         amount *= 0.3
                    when "pinch"
                         amount *= 0.1
                    when "piece", "pieces"
                         amount *= 1.0
                    else raise( ArgumentError, "Unknown metric: #{str}" )
               end
               
               @ingredient_quantities << amount
          
          else raise( ArgumentError, "Invalid amount metric in definition of ingredient #{name}" )
          end
     end
     
     def method_missing(methodName, *args, &block)
          add_ingredient(*args, &block)
     end
     
     def to_s
          result = "\n#{@name}\n===\nNutritional composition\n"
          totalEnergeticValue = 0
          
          @ingredients.length.times do |i|
               n = @ingredients[i].nutrient_quantities
               energeticValue = @ingredients[i].energetic_value * @ingredient_quantities[i]
               
               totalEnergeticValue += energeticValue
               result += "#{@ingredients[i].name}\t\t\t#{n[0]}\t#{n[1]}\t#{n[2]}\t#{energeticValue}\n"
          end
          
          result += "total energetic value\t\t\t\t#{totalEnergeticValue}\n"

     end
end